#############
# Functions #
#############
get_mean_ci <- function(x, type, log=FALSE) {
  
  if (log) {
    x <- log(x)
  }
  
  # run t test
  ttest <- t.test(x)
  
  # get statistics
  est <- ttest$estimate
  lower <- ttest$conf.int[1]
  upper <- ttest$conf.int[2]
  
  res <- c(est, lower, upper)
  names(res) <- NULL
  
  if (log) {
    if (type == "point") {return(exp(res[1]))}
    if (type == "lower") {return(exp(res[2]))}
    if (type == "upper") {return(exp(res[3]))}
  } else {
    if (type == "point") {return(res[1])}
    if (type == "lower") {return(res[2])}
    if (type == "upper") {return(res[3])}
  }
  
}

##########
# Import #
##########
# data includes antibody information for all vaccinated study participants


#####################
# Separate Datasets #
#####################
# separate
data_igg <- data %>% filter(value_name == "igg")
data_iga <- data %>% filter(value_name == "iga")
data_neut <- data %>% filter(value_name == "neut")
data_tcells <- data %>% filter(value_name == "tcell")


# log the values
data_igg$log_value <- log(data_igg$value, base = 10)
data_iga$log_value <- log(data_iga$value, base = 10)
data_neut$log_value <- log(data_neut$value, base = 2)
data_tcells$log_value <- log(data_tcells$value, base = 2)


##############
# plot Crude #
##############
plot_one <- function(this_data, ylim, title) {
  
  means_df <- 
    this_data %>% 
    filter(!is.na(encounter)) %>% 
    group_by(type, encounter) %>% 
    summarize(
      n = n(),
      point = get_mean_ci(value, "point", log=T), 
      lower = get_mean_ci(value, "lower", log=T),
      upper = get_mean_ci(value, "upper", log=T),
      .groups = "drop"
    ) %>% 
    ungroup() %>% 
    inner_join(enc_days_df, by = "encounter") 
  
  print(means_df)
  
  p <- 
    ggplot(mapping = aes(color = type)) +
    geom_point(data = this_data, mapping = aes(days, value), alpha = 0.1) +
    geom_point(data = means_df, mapping = aes(days, point), position=position_dodge(width=2)) +
    geom_errorbar(data = means_df, mapping = aes(days, ymin = lower, ymax = upper), 
                  size = 1, width = 2, position=position_dodge(width=2)) +
    theme_bw() +
    scale_x_continuous(breaks = seq(0,100,10)) +
    scale_y_continuous(trans='log10') +
    scale_color_manual(values = c("royalblue4", "darkorange3")) +
    theme(legend.title=element_blank()) +
    xlab("Days") +
    ylab(title)
  
  if (!missing(ylim)) {
    p <- p + coord_cartesian(ylim = ylim)
  }
  
  p
  
}

plot_one(data_igg, c(10,10000), "IgG (BAU, Log Scale)")
plot_one(data_iga, NULL, "IgA (S/CO, Log Scale)")
plot_one(data_neut, c(8, 2^15), "Neutralizing Antibodies (Titer, Log Scale)")
plot_one(data_tcells, NULL, "Activated T Cells (Per 10^6 PBMC, Log Scale)")

###########################
# Prepare Data for Models #
###########################
flatten <- function(this_data, this_end_max_period) {
  
  new_data <- 
    rbind(
      this_data %>% 
        select(number, days, log_value) %>% 
        filter(days <= this_end_max_period) %>% 
        group_by(number) %>% 
        summarize(log_value = max(log_value)) %>% 
        mutate(days = 0) %>% 
        inner_join(covars, by = "number")
      ,
      this_data %>% 
        filter(days > this_end_max_period) %>% 
        mutate(days = days - this_end_max_period) %>% 
        select(number, log_value, days, age, sex, bmi, Immunosupressed, number_comorb, type)
    ) %>% 
    mutate(weeks = days / 7)
  
}

data_igg_model <- flatten(data_igg, end_max_period)
data_iga_model <- flatten(data_iga, end_max_period)
data_neut_model <- flatten(data_neut, end_max_period)
data_tcells_model <- flatten(data_tcells, end_max_period)

#######
# Fit #
#######
fit_one <- function(this_data, interaction=F, base) {
  
  if (interaction) {
    
    this_fit <- lmer(log_value ~
                       weeks
                     + type
                     + weeks:type
                     + age
                     + sex
                     + bmi
                     + Immunosupressed
                     + number_comorb
                     + (1|number)
                     ,data = this_data, 
                     control=lmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))
    
  } else {
    
    this_fit <- lmer(log_value ~
                       weeks
                     + age
                     + sex
                     + bmi
                     + Immunosupressed
                     + number_comorb
                     + (1|number)
                     ,data = this_data, 
                     control=lmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))
    
  }
  
  tidy(this_fit, conf.int = T) %>% 
    transmute(effect = effect,
              group = group,
              term = term,
              estimate = base^estimate,
              conf.low = base^conf.low,
              conf.high = base^conf.high) %>% 
    mutate_if(is.numeric, round, 3)
  
}

# IgG
fit_one(data_igg_model, interaction = T, base = 10)
fit_one(data_igg_model %>% filter(type == "BNT162b2"), interaction = F, base = 10)
fit_one(data_igg_model %>% filter(type == "mRNA1273"), interaction = F, base = 10)

# IgA
fit_one(data_iga_model, interaction = T, base = 10)
fit_one(data_iga_model %>% filter(type == "BNT162b2"), interaction = F, base = 10)
fit_one(data_iga_model %>% filter(type == "mRNA1273"), interaction = F, base = 10)

# Neut
fit_one(data_neut_model, interaction = T, base = 2)
fit_one(data_neut_model %>% filter(type == "BNT162b2"), interaction = F, base = 2)
fit_one(data_neut_model %>% filter(type == "mRNA1273"), interaction = F, base = 2)

# T cells
fit_one(data_tcells_model, interaction = T, base = 2)
fit_one(data_tcells_model %>% filter(type == "BNT162b2"), interaction = F, base = 2)
fit_one(data_tcells_model %>% filter(type == "mRNA1273"), interaction = F, base = 2)

